#!/bin/bash
#
# ssh-api:ssh-common-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required." >&2
  echo "ERROR in '${BASH_SOURCE[0]}' - source message-api.sh" >&2
  exit 100
fi
if [ "$( type -t is_variable_define_and_not_empty_safe )" != 'function' ] ; then
  echofatal "ERROR in '${BASH_SOURCE[0]}' - project https://git.cclaude.rocks/bash-apis/bash-common-api.git is required."
  echofatal "ERROR in '${BASH_SOURCE[0]}' - source bash-common-api.sh"
  exit 100
fi
if [ "$( type -t getRSYNC_RESOLV_HOST )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/rsync-config-api.sh" || exit $?
fi
if ! command -v host >/dev/null ; then
  echofatal "ERROR in '${BASH_SOURCE[0]}' - host command is required."
  echofatal "ERROR in '${BASH_SOURCE[0]}' - sudo apt install dnsutils"
  exit 100
fi

function ssh_api_print_env {
  local -ri __line_max__=150

  env --null | sort -z | while IFS= read -r -d $'\0' __line__ ; do
    fixed_line="$(
      sed -z 's/\n/,/g;s/,$/\n/' <<<"${__line__}" |
        head -c "${__line_max__}"
        )"

    # printf crops the trailing newline, then add one to be sure to have only one newline
    printf '%s\n' "${fixed_line}"
  done
}

function ssh_api_get_SSH_OPTS {
  if [ -n "${SSH_API_SSH_OPTS:-}" ] ; then
    echo "${SSH_API_SSH_OPTS}"
  else
    #
    # https://stackoverflow.com/questions/16045547/why-ssh-breaks-the-control-flow-in-bash
    #
    # -n Redirects stdin from /dev/null (actually, prevents reading from stdin).
    #
    echo '-n'
  fi
}

function ssh_api_is_SSH_API_SSH_DEBUG {
  if [ "${SSH_API_SSH_DEBUG:-}" = 'true' ] ; then
    return 0 # 'true'
  else
    return 1 # 'false'
  fi
}

function _ssh_api_get_user_from_hostname {
  local -r hostname="$1"

  local user_varname=
  user_varname="$(
    tr '.' '_' <<<"${hostname}" |
    tr '-' '_' |
    tr '[:lower:]' '[:upper:]'
    )_SSH_USER"
  local -r user_varname

  if is_variable_define_and_not_empty_safe "${user_varname}" ; then
   echo "${!user_varname}"
  else
    echowarn "${FUNCNAME[0]}]( '${hostname}' ) - '${user_varname}' is not define"
    return 100
  fi
}

function ssh_api_initialize_SSH_USER_or_exit {
  local -r _ssh_host_="$1"

  SSH_USER="$( _ssh_api_get_user_from_hostname "${_ssh_host_}" )" || return $?
  declare -gr SSH_USER

  if [ -z "${SSH_USER}" ] ; then
    echofatal "SSH_USER is not define" >&2
    exit 1
  fi
}

function ssh_api_test_remote_directory {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_remote_dir_="$3"

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "${FUNCNAME[0]}() - _ssh_user_='${_ssh_user_}'"
    echodebug "${FUNCNAME[0]}() - _ssh_host_='${_ssh_host_}'"
    echodebug "${FUNCNAME[0]}() - _ssh_remote_dir_='${_ssh_remote_dir_}'"
  fi

  _ssh_api_do_ssh "${_ssh_user_}" "${_ssh_host_}" \
    "if [ -d '${_ssh_remote_dir_}' ] ; then echo 'FOUND' ; else echo 'NOTEXISTS' ; fi" || return $?
}

# Try to use resolved IP instead of HOSTNAME
function _ssh_api_common_get_ssh_host_ip_ {
  local -r _ssh_host_="$1"
  local _ssh_ip_host_=

  if [ "$( getRSYNC_RESOLV_HOST )" != 'false' ] ; then

    if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
      echodebug "${FUNCNAME[0]}() - host '${_ssh_host_}' | rev | cut -d ' ' -f1 | rev"
      echodebug "$( host "${_ssh_host_}" | rev | cut -d ' ' -f1 | rev )"
    fi

    local _host_unsafe_result_=
    if _host_unsafe_result_="$( host "${_ssh_host_}" | rev | cut -d ' ' -f1 | rev )" ; then
      _ssh_ip_host_="${_host_unsafe_result_}"

      if [ "${_ssh_ip_host_: -1}" = '.' ] ; then
        # Last character of ${_ssh_ip_host_} is '.'
        _ssh_ip_host_="${_ssh_host_}" # result is a domain, assume ${_ssh_host_} is already an IP
      fi
    fi
  fi

  if [ -n "${_ssh_ip_host_}" ] ; then
    echo "${_ssh_ip_host_}"
  else
    echo "${_ssh_host_}"
  fi
}

function _ssh_api_do_ssh {
  local -r _ssh_user_="$1"
  local -r _ssh_host_="$2"
  local -r _ssh_cmd_="$3"

  local _ssh_ip_host_=
  _ssh_ip_host_="$( _ssh_api_common_get_ssh_host_ip_ "${_ssh_host_}" )" || return $?
  local -r _ssh_ip_host_

  local _ssh_opts_=
  _ssh_opts_="$( ssh_api_get_SSH_OPTS )" || return $?
  local -r _ssh_opts_

  local -i ssh_rc=0

  (
    if ssh_api_is_SSH_API_SSH_DEBUG ; then
      set -x
    fi

    # shellcheck disable=SC2029 disable=SC2086 # expands script on the client side, Double quote to...
    ssh ${_ssh_opts_} "${_ssh_user_}@${_ssh_ip_host_}" "${_ssh_cmd_}"
  ) || ssh_rc=$?

  if [ "${ssh_rc}" -ne 0 ] ; then
    echofatal "${FUNCNAME[0]}() - ssh error with ${ssh_rc} running with '${USER:-$( id -u -n )}' from: '$( pwd )'"
    echofatal "${FUNCNAME[0]}() - USER: '${USER:-}' / id -u -n: '$( id -u -n )'"
    echofatal "ssh access: '${_ssh_user_}@${_ssh_ip_host_}'"
    echofatal "ssh resolution: _ssh_ip_host_='${_ssh_ip_host_}'"
    echofatal "ssh resolution: _ssh_host_='${_ssh_host_}'"
    echofatal "SSH_AGENT_PID=${SSH_AGENT_PID:-}"
    echofatal "SSH_AUTH_SOCK=${SSH_AUTH_SOCK:-}"
    echofatal "ssh command: ${_ssh_cmd_}"
    echofatal "stacktrace: ${FUNCNAME[*]}"

    if ssh_api_is_SSH_API_SSH_DEBUG ; then
      echofatal 'env'
      ssh_api_print_env >&2
    fi

    return "${ssh_rc}"
  fi
}

function ssh_api_start_ssh_agent {
  if [ -z "${SSH_AUTH_SOCK:-}" ] || [ -z "${SSH_AGENT_PID:-}" ] ; then
    echowarn 'start ssh-agent'

    # shellcheck disable=SC2046 # Quote this to prevent word splitting.
    eval $( ssh-agent -s )
    # Now SSH_AUTH_SOCK and SSH_AGENT_PID defined (and exported)

    if [ "${SSH_API_DISABLED_SSH_AGENT_ADD_PRIVATE_KEY:-}" != 'true' ] ; then
      # adds private key identities
      echowarn 'Add default keys'
      ssh-add
    fi
  fi
}

if [ "${SSH_API_DISABLED_SSH_AGENT_AUTO_START:-}" != 'true' ] ; then
  ssh_api_start_ssh_agent >&2
fi
